==> simpapy-0.0.4a0.tar.gz: simpapy-0.0.4a0/tests/test_SIMPA.py <==
import os
import pandas as pd
import numpy as np
import pytest
import unittest
from unittest.mock import patch, mock_open, MagicMock, Mock, call
from SIMPApy.SIMPA import simpa, calculate_wcos_mpes, sort_gene_list, run_simpa_batch, load_simpa
import glob

@pytest.fixture
def mock_rna_data():
    """Create mock RNA GSEA data"""
    return pd.DataFrame({
        'Term': ['Pathway1', 'Pathway2', 'Pathway3'],
        'pval': [0.01, 0.001, 0.05],
        'fdr': [0.03, 0.01, 0.1],
        'nes': [1.5, 2.0, -0.8],
        'lead_genes': ['GeneA;GeneB', 'GeneC', 'GeneD;GeneE;GeneF'],
        'matched_genes': ['GeneA;GeneB;GeneC', 'GeneC;GeneD', 'GeneD;GeneE;GeneF;GeneG']
    })

@pytest.fixture
def mock_cnv_data():
    """Create mock CNV GSEA data"""
    return pd.DataFrame({
        'Term': ['Pathway1', 'Pathway2', 'Pathway3'],
        'pval': [0.02, 0.003, 0.04],
        'fdr': [0.04, 0.02, 0.09],
        'nes': [1.3, 1.8, -0.7],
        'lead_genes': ['GeneA', 'GeneC;GeneD', 'GeneE'],
        'matched_genes': ['GeneA;GeneB;GeneC', 'GeneC;GeneD', 'GeneD;GeneE;GeneF;GeneG']
    })

@pytest.fixture
def mock_dna_data():
    """Create mock DNA methylation GSEA data"""
    return pd.DataFrame({
        'Term': ['Pathway1', 'Pathway2', 'Pathway3'],
        'pval': [0.03, 0.008, 0.02],
        'fdr': [0.05, 0.03, 0.06],
        'nes': [1.2, 1.6, -1.0],
        'lead_genes': ['GeneB', 'GeneC', 'GeneF'],
        'matched_genes': ['GeneA;GeneB;GeneC', 'GeneC;GeneD', 'GeneD;GeneE;GeneF;GeneG']
    })

def test_simpa_successful_integration(mock_rna_data, mock_cnv_data, mock_dna_data):
    """Test successful integration of GSEA results"""
    # Mock the read_csv function to return our test data
    with patch('pandas.read_csv') as mock_read_csv:
        mock_read_csv.side_effect = [mock_rna_data, mock_cnv_data, mock_dna_data]
        
        # Run simpa function
        result = simpa(
            sample_id='test_sample',
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna'
        )
        
        # Verify basic properties of the result
        assert result is not None
        assert isinstance(result, pd.DataFrame)
        assert len(result) == 3  # All pathways should be present
        assert 'Term' in result.columns
        assert 'combined_pval' in result.columns
        assert 'mpes' in result.columns

def test_simpa_missing_file():
    """Test handling of missing files"""
    with patch('pandas.read_csv') as mock_read_csv:
        mock_read_csv.side_effect = FileNotFoundError
        
        result = simpa(
            sample_id='missing_sample',
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna'
        )
        
        assert result is None

def test_simpa_empty_file():
    """Test handling of empty files"""
    with patch('pandas.read_csv') as mock_read_csv:
        mock_read_csv.side_effect = pd.errors.EmptyDataError
        
        result = simpa(
            sample_id='empty_sample',
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna'
        )
        
        assert result is None

def test_simpa_output_file(mock_rna_data, mock_cnv_data, mock_dna_data):
    """Test that results are saved to a file when output_dir is provided"""
    with patch('pandas.read_csv') as mock_read_csv, \
         patch('os.makedirs') as mock_makedirs, \
         patch('pandas.DataFrame.to_csv') as mock_to_csv:
        
        mock_read_csv.side_effect = [mock_rna_data, mock_cnv_data, mock_dna_data]
        
        simpa(
            sample_id='test_sample',
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna',
            output_dir='/fake/output'
        )
        
        # Verify that the directory was created and to_csv was called
        mock_makedirs.assert_called_once_with('/fake/output', exist_ok=True)
        mock_to_csv.assert_called_once()

def test_simpa_calculation_correctness(mock_rna_data, mock_cnv_data, mock_dna_data):
    """Test that calculations in simpa are correct"""
    with patch('pandas.read_csv') as mock_read_csv:
        mock_read_csv.side_effect = [mock_rna_data, mock_cnv_data, mock_dna_data]
        
        result = simpa(
            sample_id='test_sample',
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna'
        )
        
        # Check that z-scores are calculated correctly
        # For pathway1, RNA p-value is 0.01, which corresponds to z ≈ 2.33
        # Sign should match NES (1.5), so z ≈ 2.33
        pathway1 = result[result['Term'] == 'Pathway1'].iloc[0]
        assert round(pathway1['combined_z'], 2) != 0  # Should not be zero
        
        # Check that WCOS and MPES are calculated
        assert 'rna_wcos' in result.columns
        assert 'cnv_wcos' in result.columns
        assert 'dna_wcos' in result.columns
        assert 'mpes' in result.columns
        
        # Verify that all WCOS values are not NaN
        assert not result['rna_wcos'].isna().any()
        assert not result['cnv_wcos'].isna().any()
        assert not result['dna_wcos'].isna().any()

def test_sort_gene_list():
    """Test the sort_gene_list function"""
    assert sort_gene_list("GeneC;GeneA;GeneB") == "GeneA;GeneB;GeneC"
    assert sort_gene_list("") == ""
    assert sort_gene_list(None) is None
    assert sort_gene_list(np.nan) is np.nan

def test_calculate_wcos_mpes():
    """Test the calculate_wcos_mpes function"""
    row = pd.Series({
        'rna_fdr': 0.01,
        'cnv_fdr': 0.02,
        'dna_fdr': 0.03,
        'rna_nes': 1.5,
        'cnv_nes': 1.2,
        'dna_nes': 0.8,
        'rna_lead_genes': 'GeneA;GeneB',
        'cnv_lead_genes': 'GeneA',
        'dna_lead_genes': 'GeneB;GeneC',
        'matched_genes': 'GeneA;GeneB;GeneC;GeneD'
    })
    
    result = calculate_wcos_mpes(row)
    
    assert isinstance(result, pd.Series)
    assert 'rna_wcos' in result.index
    assert 'cnv_wcos' in result.index
    assert 'dna_wcos' in result.index
    assert 'mpes' in result.index
    
    # WCOS should be positive for positive NES
    assert result['rna_wcos'] > 0
    assert result['cnv_wcos'] > 0
    assert result['dna_wcos'] > 0


@pytest.fixture
def mock_simpa_result():
    """Create a mock DataFrame that would be returned by the simpa function"""
    return pd.DataFrame({
        'Term': ['Pathway1', 'Pathway2'],
        'combined_pval': [0.01, 0.02],
        'combined_z': [2.5, 2.0],
        'fdr_bh': [0.03, 0.05],
        'matched_genes': ['GeneA;GeneB;GeneC', 'GeneD;GeneE'],
        'rna_lead_genes': ['GeneA;GeneB', 'GeneD'],
        'cnv_lead_genes': ['GeneA', 'GeneD;GeneE'],
        'dna_lead_genes': ['GeneB', 'GeneE'],
        'rna_nes': [1.5, 1.2],
        'cnv_nes': [1.3, 1.1],
        'dna_nes': [0.8, 0.9],
        'rna_wcos': [0.5, 0.4],
        'cnv_wcos': [0.4, 0.3],
        'dna_wcos': [0.3, 0.2],
        'mpes': [1.8, 1.5]
    })

def test_run_simpa_batch_successful_execution(mock_simpa_result):
    """Test that run_simpa_batch successfully processes multiple samples"""
    sample_ids = ['sample1', 'sample2', 'sample3']
    
    with patch('os.makedirs') as mock_makedirs, \
         patch('SIMPApy.SIMPA.simpa', return_value=mock_simpa_result) as mock_simpa, \
         patch('pandas.DataFrame.to_csv') as mock_to_csv:
        
        run_simpa_batch(
            sample_ids=sample_ids,
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna',
            output_dir='/fake/output'
        )
        
        # Check that output directory was created
        mock_makedirs.assert_called_once_with('/fake/output', exist_ok=True)
        
        # Check that simpa was called for each sample with correct parameters
        assert mock_simpa.call_count == 3
        mock_simpa.assert_has_calls([
            call('sample1', '/fake/rna', '/fake/cnv', '/fake/dna'),
            call('sample2', '/fake/rna', '/fake/cnv', '/fake/dna'),
            call('sample3', '/fake/rna', '/fake/cnv', '/fake/dna')
        ])
        
        # Check that to_csv was called for each sample
        assert mock_to_csv.call_count == 3

def test_run_simpa_batch_with_failed_sample(mock_simpa_result):
    """Test that run_simpa_batch continues even if one sample fails"""
    sample_ids = ['sample1', 'sample2']
    
    # Make simpa return None for the first sample (simulating failure)
    def side_effect(sample_id, *args):
        if sample_id == 'sample1':
            return None
        return mock_simpa_result
    
    with patch('os.makedirs') as mock_makedirs, \
         patch('SIMPApy.SIMPA.simpa', side_effect=side_effect) as mock_simpa, \
         patch('pandas.DataFrame.to_csv') as mock_to_csv:
        
        run_simpa_batch(
            sample_ids=sample_ids,
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna',
            output_dir='/fake/output'
        )
        
        # Check that simpa was called for each sample
        assert mock_simpa.call_count == 2
        
        # Check that to_csv was called only for the successful sample
        assert mock_to_csv.call_count == 1

def test_load_simpa_successful_loading():
    """Test successful loading of SIMPA results"""
    # Create mock file data and paths
    mock_files = {
        '/fake/results/tm1_integrated_gsea_results.csv': pd.DataFrame({
            'Term': ['Pathway1', 'Pathway2'],
            'combined_pval': [0.01, 0.02],
            'combined_z': [2.5, 2.0],
            'fdr_bh': [0.03, 0.05],
            'matched_genes': ['GeneA;GeneB', 'GeneC;GeneD'],
            'rna_lead_genes': ['GeneA', 'GeneC'],
            'cnv_lead_genes': ['GeneB', 'GeneD'],
            'dna_lead_genes': ['GeneA', 'GeneC'],
            'rna_nes': [1.5, 1.2],
            'cnv_nes': [1.3, 1.1],
            'dna_nes': [0.8, 0.9],
            'mpes': [1.8, 1.5]
        }),
        '/fake/results/tw2_integrated_gsea_results.csv': pd.DataFrame({
            'Term': ['Pathway1', 'Pathway3'],
            'combined_pval': [0.03, 0.04],
            'combined_z': [1.5, 1.0],
            'fdr_bh': [0.06, 0.08],
            'matched_genes': ['GeneA;GeneB', 'GeneE;GeneF'],
            'rna_lead_genes': ['GeneA', 'GeneF'],
            'cnv_lead_genes': ['GeneB', 'GeneE'],
            'dna_lead_genes': ['GeneA', 'GeneF'],
            'rna_nes': [1.1, 0.9],
            'cnv_nes': [1.0, 0.8],
            'dna_nes': [0.7, 0.6],
            'mpes': [1.4, 1.2]
        })
    }
    
    # Mock glob.glob to return our file paths
    def mock_glob_side_effect(pattern):
        if 'tm*' in pattern:
            return ['/fake/results/tm1_integrated_gsea_results.csv']
        elif 'tw*' in pattern:
            return ['/fake/results/tw2_integrated_gsea_results.csv']
        return []
    
    # Mock pd.read_csv to return our mock data based on file path
    def mock_read_csv_side_effect(file_path):
        return mock_files[file_path]
    
    with patch('glob.glob', side_effect=mock_glob_side_effect), \
         patch('pandas.read_csv', side_effect=mock_read_csv_side_effect):
        
        result = load_simpa('/fake/results')
        
        # Check that the result has the right structure
        assert isinstance(result, pd.DataFrame)
        assert len(result) == 4  # 2 rows from each file
        
        # Check that sample names were added correctly
        assert all(result[result['sample_name'] == 'tm1']['Term'].isin(['Pathway1', 'Pathway2']))
        assert all(result[result['sample_name'] == 'tw2']['Term'].isin(['Pathway1', 'Pathway3']))
        
        # Check that we have all the expected columns
        expected_columns = ['Term', 'combined_pval', 'combined_z', 'fdr_bh', 'matched_genes',
                           'rna_lead_genes', 'cnv_lead_genes', 'dna_lead_genes', 
                           'rna_nes', 'cnv_nes', 'dna_nes', 'mpes', 'sample_name']
        for col in expected_columns:
            assert col in result.columns

def test_load_simpa_with_parse_error():
    """Test that load_simpa handles parsing errors gracefully"""
    # Mock glob.glob to return our file paths
    def mock_glob_side_effect(pattern):
        if 'tm*' in pattern:
            return ['/fake/results/tm1_integrated_gsea_results.csv', 
                    '/fake/results/tm2_integrated_gsea_results.csv']
        return []
    
    # Mock pd.read_csv to succeed for the first file and fail with ParserError for the second
    def mock_read_csv_side_effect(file_path):
        if file_path == '/fake/results/tm1_integrated_gsea_results.csv':
            return pd.DataFrame({
                'Term': ['Pathway1'],
                'combined_pval': [0.01],
                'combined_z': [2.5],
                'fdr_bh': [0.03],
                'matched_genes': ['GeneA;GeneB'],
                'rna_lead_genes': ['GeneA'],
                'cnv_lead_genes': ['GeneB'],
                'dna_lead_genes': ['GeneA'],
                'rna_nes': [1.5],
                'cnv_nes': [1.3],
                'dna_nes': [0.8],
                'mpes': [1.8]
            })
        else:
            raise pd.errors.ParserError("Mocked parsing error")
    
    with patch('glob.glob', side_effect=mock_glob_side_effect), \
         patch('pandas.read_csv', side_effect=mock_read_csv_side_effect), \
         patch('builtins.print') as mock_print:
        
        result = load_simpa('/fake/results')
        
        # Check that we still got results from the first file
        assert isinstance(result, pd.DataFrame)
        assert len(result) == 1
        assert result['sample_name'].iloc[0] == 'tm1'
        
        # Check that the error was printed
        mock_print.assert_called_once_with(
            "Error: Could not parse /fake/results/tm2_integrated_gsea_results.csv as a CSV file. Skipping.")

@unittest.expectedFailure
def test_load_simpa_empty_directory_fixed():
    """Test load_simpa with an empty directory (fixed version)"""
    # Mock glob.glob to return empty lists (no files found)
    with patch('glob.glob', return_value=[]):
        try:
            result = load_simpa('/fake/empty')
            # If it succeeds, verify it returned an empty DataFrame
            assert isinstance(result, pd.DataFrame)
            assert len(result) == 0
        except ValueError as e:
            # If it fails with ValueError, verify it's the expected error
            assert "No objects to concatenate" in str(e)
            # This is the expected behavior currently
            pass

==> simpapy-0.1.0.tar.gz: simpapy-0.1.0/tests/test_SIMPA.py <==
import os
import pandas as pd
import numpy as np
import pytest
import unittest
from unittest.mock import patch, mock_open, MagicMock, Mock, call
from SIMPApy.SIMPA import _simpa, calculate_wcos_mpes, sort_gene_list, simpa, load_simpa
import glob

@pytest.fixture
def mock_rna_data():
    """Create mock RNA GSEA data"""
    return pd.DataFrame({
        'Term': ['Pathway1', 'Pathway2', 'Pathway3'],
        'pval': [0.01, 0.001, 0.05],
        'fdr': [0.03, 0.01, 0.1],
        'nes': [1.5, 2.0, -0.8],
        'lead_genes': ['GeneA;GeneB', 'GeneC', 'GeneD;GeneE;GeneF'],
        'matched_genes': ['GeneA;GeneB;GeneC', 'GeneC;GeneD', 'GeneD;GeneE;GeneF;GeneG']
    })

@pytest.fixture
def mock_cnv_data():
    """Create mock CNV GSEA data"""
    return pd.DataFrame({
        'Term': ['Pathway1', 'Pathway2', 'Pathway3'],
        'pval': [0.02, 0.003, 0.04],
        'fdr': [0.04, 0.02, 0.09],
        'nes': [1.3, 1.8, -0.7],
        'lead_genes': ['GeneA', 'GeneC;GeneD', 'GeneE'],
        'matched_genes': ['GeneA;GeneB;GeneC', 'GeneC;GeneD', 'GeneD;GeneE;GeneF;GeneG']
    })

@pytest.fixture
def mock_dna_data():
    """Create mock DNA methylation GSEA data"""
    return pd.DataFrame({
        'Term': ['Pathway1', 'Pathway2', 'Pathway3'],
        'pval': [0.03, 0.008, 0.02],
        'fdr': [0.05, 0.03, 0.06],
        'nes': [1.2, 1.6, -1.0],
        'lead_genes': ['GeneB', 'GeneC', 'GeneF'],
        'matched_genes': ['GeneA;GeneB;GeneC', 'GeneC;GeneD', 'GeneD;GeneE;GeneF;GeneG']
    })

def test_simpa_successful_integration(mock_rna_data, mock_cnv_data, mock_dna_data):
    """Test successful integration of GSEA results"""
    # Mock the read_csv function to return our test data
    with patch('pandas.read_csv') as mock_read_csv:
        mock_read_csv.side_effect = [mock_rna_data, mock_cnv_data, mock_dna_data]
        
        # Run simpa function
        result = _simpa(
            sample_id='test_sample',
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna'
        )
        
        # Verify basic properties of the result
        assert result is not None
        assert isinstance(result, pd.DataFrame)
        assert len(result) == 3  # All pathways should be present
        assert 'Term' in result.columns
        assert 'combined_pval' in result.columns
        assert 'mpes' in result.columns

def test_simpa_missing_file():
    """Test handling of missing files"""
    with patch('pandas.read_csv') as mock_read_csv:
        mock_read_csv.side_effect = FileNotFoundError
        
        result = _simpa(
            sample_id='missing_sample',
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna'
        )
        
        assert result is None

def test_simpa_empty_file():
    """Test handling of empty files"""
    with patch('pandas.read_csv') as mock_read_csv:
        mock_read_csv.side_effect = pd.errors.EmptyDataError
        
        result = _simpa(
            sample_id='empty_sample',
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna'
        )
        
        assert result is None

def test_simpa_output_file(mock_rna_data, mock_cnv_data, mock_dna_data):
    """Test that results are saved to a file when output_dir is provided"""
    with patch('pandas.read_csv') as mock_read_csv, \
         patch('os.makedirs') as mock_makedirs, \
         patch('pandas.DataFrame.to_csv') as mock_to_csv:
        
        mock_read_csv.side_effect = [mock_rna_data, mock_cnv_data, mock_dna_data]
        
        _simpa(
            sample_id='test_sample',
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna',
            output_dir='/fake/output'
        )
        
        # Verify that the directory was created and to_csv was called
        mock_makedirs.assert_called_once_with('/fake/output', exist_ok=True)
        mock_to_csv.assert_called_once()

def test_simpa_calculation_correctness(mock_rna_data, mock_cnv_data, mock_dna_data):
    """Test that calculations in simpa are correct"""
    with patch('pandas.read_csv') as mock_read_csv:
        mock_read_csv.side_effect = [mock_rna_data, mock_cnv_data, mock_dna_data]
        
        result = _simpa(
            sample_id='test_sample',
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna'
        )
        
        # Check that z-scores are calculated correctly
        # For pathway1, RNA p-value is 0.01, which corresponds to z ≈ 2.33
        # Sign should match NES (1.5), so z ≈ 2.33
        pathway1 = result[result['Term'] == 'Pathway1'].iloc[0]
        assert round(pathway1['combined_z'], 2) != 0  # Should not be zero
        
        # Check that WCOS and MPES are calculated
        assert 'rna_wcos' in result.columns
        assert 'cnv_wcos' in result.columns
        assert 'dna_wcos' in result.columns
        assert 'mpes' in result.columns
        
        # Verify that all WCOS values are not NaN
        assert not result['rna_wcos'].isna().any()
        assert not result['cnv_wcos'].isna().any()
        assert not result['dna_wcos'].isna().any()

def test_sort_gene_list():
    """Test the sort_gene_list function"""
    assert sort_gene_list("GeneC;GeneA;GeneB") == "GeneA;GeneB;GeneC"
    assert sort_gene_list("") == ""
    assert sort_gene_list(None) is None
    assert sort_gene_list(np.nan) is np.nan

def test_calculate_wcos_mpes():
    """Test the calculate_wcos_mpes function"""
    row = pd.Series({
        'rna_fdr': 0.01,
        'cnv_fdr': 0.02,
        'dna_fdr': 0.03,
        'rna_nes': 1.5,
        'cnv_nes': 1.2,
        'dna_nes': 0.8,
        'rna_lead_genes': 'GeneA;GeneB',
        'cnv_lead_genes': 'GeneA',
        'dna_lead_genes': 'GeneB;GeneC',
        'matched_genes': 'GeneA;GeneB;GeneC;GeneD'
    })
    
    result = calculate_wcos_mpes(row)
    
    assert isinstance(result, pd.Series)
    assert 'rna_wcos' in result.index
    assert 'cnv_wcos' in result.index
    assert 'dna_wcos' in result.index
    assert 'mpes' in result.index
    
    # WCOS should be positive for positive NES
    assert result['rna_wcos'] > 0
    assert result['cnv_wcos'] > 0
    assert result['dna_wcos'] > 0


@pytest.fixture
def mock_simpa_result():
    """Create a mock DataFrame that would be returned by the simpa function"""
    return pd.DataFrame({
        'Term': ['Pathway1', 'Pathway2'],
        'combined_pval': [0.01, 0.02],
        'combined_z': [2.5, 2.0],
        'fdr_bh': [0.03, 0.05],
        'matched_genes': ['GeneA;GeneB;GeneC', 'GeneD;GeneE'],
        'rna_lead_genes': ['GeneA;GeneB', 'GeneD'],
        'cnv_lead_genes': ['GeneA', 'GeneD;GeneE'],
        'dna_lead_genes': ['GeneB', 'GeneE'],
        'rna_nes': [1.5, 1.2],
        'cnv_nes': [1.3, 1.1],
        'dna_nes': [0.8, 0.9],
        'rna_wcos': [0.5, 0.4],
        'cnv_wcos': [0.4, 0.3],
        'dna_wcos': [0.3, 0.2],
        'mpes': [1.8, 1.5]
    })

def test_run_simpa_batch_successful_execution(mock_simpa_result):
    """Test that run_simpa_batch successfully processes multiple samples"""
    sample_ids = ['sample1', 'sample2', 'sample3']
    
    with patch('os.makedirs') as mock_makedirs, \
         patch('SIMPApy.SIMPA._simpa', return_value=mock_simpa_result) as mock_simpa, \
         patch('pandas.DataFrame.to_csv') as mock_to_csv:
        
        simpa(
            sample_ids=sample_ids,
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna',
            output_dir='/fake/output'
        )
        
        # Check that output directory was created
        mock_makedirs.assert_called_once_with('/fake/output', exist_ok=True)
        
        # Check that simpa was called for each sample with correct parameters
        assert mock_simpa.call_count == 3
        mock_simpa.assert_has_calls([
            call('sample1', '/fake/rna', '/fake/cnv', '/fake/dna'),
            call('sample2', '/fake/rna', '/fake/cnv', '/fake/dna'),
            call('sample3', '/fake/rna', '/fake/cnv', '/fake/dna')
        ])
        
        # Check that to_csv was called for each sample
        assert mock_to_csv.call_count == 3

def test_run_simpa_batch_with_failed_sample(mock_simpa_result):
    """Test that run_simpa_batch continues even if one sample fails"""
    sample_ids = ['sample1', 'sample2']
    
    # Make simpa return None for the first sample (simulating failure)
    def side_effect(sample_id, *args):
        if sample_id == 'sample1':
            return None
        return mock_simpa_result
    
    with patch('os.makedirs') as mock_makedirs, \
         patch('SIMPApy.SIMPA._simpa', side_effect=side_effect) as mock_simpa, \
         patch('pandas.DataFrame.to_csv') as mock_to_csv:
        
        simpa(
            sample_ids=sample_ids,
            rna_dir='/fake/rna',
            cnv_dir='/fake/cnv',
            dna_dir='/fake/dna',
            output_dir='/fake/output'
        )
        
        # Check that simpa was called for each sample
        assert mock_simpa.call_count == 2
        
        # Check that to_csv was called only for the successful sample
        assert mock_to_csv.call_count == 1

def test_load_simpa_successful_loading():
    """Test successful loading of SIMPA results"""
    # Create mock file data and paths
    mock_files = {
        '/fake/results/tm1_integrated_gsea_results.csv': pd.DataFrame({
            'Term': ['Pathway1', 'Pathway2'],
            'combined_pval': [0.01, 0.02],
            'combined_z': [2.5, 2.0],
            'fdr_bh': [0.03, 0.05],
            'matched_genes': ['GeneA;GeneB', 'GeneC;GeneD'],
            'rna_lead_genes': ['GeneA', 'GeneC'],
            'cnv_lead_genes': ['GeneB', 'GeneD'],
            'dna_lead_genes': ['GeneA', 'GeneC'],
            'rna_nes': [1.5, 1.2],
            'cnv_nes': [1.3, 1.1],
            'dna_nes': [0.8, 0.9],
            'mpes': [1.8, 1.5]
        }),
        '/fake/results/tw2_integrated_gsea_results.csv': pd.DataFrame({
            'Term': ['Pathway1', 'Pathway3'],
            'combined_pval': [0.03, 0.04],
            'combined_z': [1.5, 1.0],
            'fdr_bh': [0.06, 0.08],
            'matched_genes': ['GeneA;GeneB', 'GeneE;GeneF'],
            'rna_lead_genes': ['GeneA', 'GeneF'],
            'cnv_lead_genes': ['GeneB', 'GeneE'],
            'dna_lead_genes': ['GeneA', 'GeneF'],
            'rna_nes': [1.1, 0.9],
            'cnv_nes': [1.0, 0.8],
            'dna_nes': [0.7, 0.6],
            'mpes': [1.4, 1.2]
        })
    }
    
    # Mock glob.glob to return our file paths
    def mock_glob_side_effect(pattern):
        if 'tm*' in pattern:
            return ['/fake/results/tm1_integrated_gsea_results.csv']
        elif 'tw*' in pattern:
            return ['/fake/results/tw2_integrated_gsea_results.csv']
        return []
    
    # Mock pd.read_csv to return our mock data based on file path
    def mock_read_csv_side_effect(file_path):
        return mock_files[file_path]
    
    with patch('glob.glob', side_effect=mock_glob_side_effect), \
         patch('pandas.read_csv', side_effect=mock_read_csv_side_effect):
        
        result = load_simpa('/fake/results')
        
        # Check that the result has the right structure
        assert isinstance(result, pd.DataFrame)
        assert len(result) == 4  # 2 rows from each file
        
        # Check that sample names were added correctly
        assert all(result[result['sample_name'] == 'tm1']['Term'].isin(['Pathway1', 'Pathway2']))
        assert all(result[result['sample_name'] == 'tw2']['Term'].isin(['Pathway1', 'Pathway3']))
        
        # Check that we have all the expected columns
        expected_columns = ['Term', 'combined_pval', 'combined_z', 'fdr_bh', 'matched_genes',
                           'rna_lead_genes', 'cnv_lead_genes', 'dna_lead_genes', 
                           'rna_nes', 'cnv_nes', 'dna_nes', 'mpes', 'sample_name']
        for col in expected_columns:
            assert col in result.columns

def test_load_simpa_with_parse_error():
    """Test that load_simpa handles parsing errors gracefully"""
    # Mock glob.glob to return our file paths
    def mock_glob_side_effect(pattern):
        if 'tm*' in pattern:
            return ['/fake/results/tm1_integrated_gsea_results.csv', 
                    '/fake/results/tm2_integrated_gsea_results.csv']
        return []
    
    # Mock pd.read_csv to succeed for the first file and fail with ParserError for the second
    def mock_read_csv_side_effect(file_path):
        if file_path == '/fake/results/tm1_integrated_gsea_results.csv':
            return pd.DataFrame({
                'Term': ['Pathway1'],
                'combined_pval': [0.01],
                'combined_z': [2.5],
                'fdr_bh': [0.03],
                'matched_genes': ['GeneA;GeneB'],
                'rna_lead_genes': ['GeneA'],
                'cnv_lead_genes': ['GeneB'],
                'dna_lead_genes': ['GeneA'],
                'rna_nes': [1.5],
                'cnv_nes': [1.3],
                'dna_nes': [0.8],
                'mpes': [1.8]
            })
        else:
            raise pd.errors.ParserError("Mocked parsing error")
    
    with patch('glob.glob', side_effect=mock_glob_side_effect), \
         patch('pandas.read_csv', side_effect=mock_read_csv_side_effect), \
         patch('builtins.print') as mock_print:
        
        result = load_simpa('/fake/results')
        
        # Check that we still got results from the first file
        assert isinstance(result, pd.DataFrame)
        assert len(result) == 1
        assert result['sample_name'].iloc[0] == 'tm1'
        
        # Check that the error was printed
        mock_print.assert_called_once_with(
            "Error: Could not parse /fake/results/tm2_integrated_gsea_results.csv as a CSV file. Skipping.")

@unittest.expectedFailure
def test_load_simpa_empty_directory_fixed():
    """Test load_simpa with an empty directory (fixed version)"""
    # Mock glob.glob to return empty lists (no files found)
    with patch('glob.glob', return_value=[]):
        try:
            result = load_simpa('/fake/empty')
            # If it succeeds, verify it returned an empty DataFrame
            assert isinstance(result, pd.DataFrame)
            assert len(result) == 0
        except ValueError as e:
            # If it fails with ValueError, verify it's the expected error
            assert "No objects to concatenate" in str(e)
            # This is the expected behavior currently
            pass
